/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file hello-world.cpp
@ingroup ExamplePrograms
@brief Shows the use of QVApllication and QVWorker classes.

@todo Detailed description should follow.

@section Usage
Simply compile, and execute the binary:
\code
	./hello-world
\endcode

@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include <qvcore/qvapplication.h>
#include <qvgui/qvgui.h>

/// \cond
class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name):QVWorker(name)
			{
			addTrigger("Trigger 1");
			addTrigger("Trigger 2");
			}

		void iterate()
			{
			std::cout << "Worker " <<
				qPrintable(this->getName()) << ", iteration "
				<< this->getIteration() << std::endl;
			}

		void processTrigger(QString triggerName)
			{
			std::cout << "QVWorker " << qPrintable(getName()) << ": trigger '"
				<< qPrintable(triggerName) << "' activated." << std::endl;
			};
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Shows the use of QVApllication and QVWorker classes."
		);

	MyWorker worker("QVWorker 'Hello world'");

	QVGUI interface;

	return app.exec();
	}

/// \endcond
