/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvmodels3D.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>

#include <models3D.h>

#define CIRCLE_NUM_SECTORS  9 /* Nmero de sectores por c�culo. */
#define PI                  3.141593
#define SQR3DIV2            0.866025
/* Colores b�icos: */
#define COLOR_PARED            220,220,220
#define COLOR_PUERTA           0,200,0
#define COLOR_MARCO            200,200,0
#define COLOR_MANIVELA         255,255,0
#define COLOR_RODAPIE          255,100,100
#define COLOR_SUELO            200,80,80
#define COLOR_CUERPO_ROBOT     255,0,0
#define COLOR_TAPA_ROBOT       10,10,10
#define COLOR_RUEDAS_ROBOT     20,20,20
#define COLOR_TRIPODE_ROBOT    100,100,100
#define COLOR_CAMARA           0,0,0

/* (Unidades en cm): */
/* Pasillo: */
#define ALTO_PARED    270 
#define ALTO_RODAPIE  20 

/* Puertas: */
//Ahora es variable#define ANCHO_PUERTA      80  /* Sin contar el marco */ 
#define ALTO_PUERTA       210 /* Sin contar el marco */
#define ANCHO_MARCO       10
#define RADIO_MANIVELA    6

/* Robot: */
#define RADIO_ROBOT            25
#define RADIO_TAPA_ROBOT       28
#define ALTO_ROBOT             20 /* Del cuerpo, sin contar elevaci�. */
#define ELEVACION_ROBOT         5 /* Elevaci� sobre el suelo. */
#define ALTO_TRIPODE           70 /* Sin contar elevaci� ni cuerpo. */
#define RADIO_INT_RUEDA_ROBOT   5 
#define RADIO_EXT_RUEDA_ROBOT  10 
#define CAM_DEPTH              15 /* Medidas de la c�ara. */
#define CAM_WIDTH               5
#define CAM_HEIGHT              8

void puerta3D_internal(float ancho)
{
    int i;
    float angle;
    float ancho_puerta=ancho-2*ANCHO_MARCO;
    
    
    glBegin(GL_QUADS); 
      /* 3 rect�gulos para el marco: */    
      glColor3ub(COLOR_MARCO);

      /* Marco izquierdo: */
      glVertex3f(0,0,0);
      glVertex3f(0,0,ALTO_PUERTA);
      glVertex3f(ANCHO_MARCO,0,ALTO_PUERTA);
      glVertex3f(ANCHO_MARCO,0,0);

      /* Marco derecho: */
      glVertex3f(ancho_puerta+ANCHO_MARCO,0,0);
      glVertex3f(ancho_puerta+ANCHO_MARCO,0,ALTO_PUERTA);
      glVertex3f(ancho_puerta+2*ANCHO_MARCO,0,ALTO_PUERTA);
      glVertex3f(ancho_puerta+2*ANCHO_MARCO,0,0);
     
      /* Marco superior: */
      glVertex3f(0,0,ALTO_PUERTA);
      glVertex3f(0,0,ALTO_PUERTA+ANCHO_MARCO);
      glVertex3f(ancho_puerta+2*ANCHO_MARCO,0,ALTO_PUERTA+ANCHO_MARCO);
      glVertex3f(ancho_puerta+2*ANCHO_MARCO,0,ALTO_PUERTA);
      
      /* Puerta: */
      glColor3ub(COLOR_PUERTA);

      glVertex3f(ANCHO_MARCO,0,0);
      glVertex3f(ANCHO_MARCO,0,ALTO_PUERTA);
      glVertex3f(ANCHO_MARCO+ancho_puerta,0,ALTO_PUERTA);
      glVertex3f(ANCHO_MARCO+ancho_puerta,0,0);
      
      /* Cuadrado de pared encima de la puerta: */
      glColor3ub(COLOR_PARED);

      glVertex3f(0,0,ALTO_PUERTA+ANCHO_MARCO);
      glVertex3f(0,0,ALTO_PARED);
      glVertex3f(2*ANCHO_MARCO+ancho_puerta,0,ALTO_PARED);
      glVertex3f(2*ANCHO_MARCO+ancho_puerta,0,ALTO_PUERTA+ANCHO_MARCO);

    glEnd();

    
    glBegin(GL_TRIANGLE_FAN);
      /* Manivela: */
      glColor3ub(COLOR_MANIVELA);

      glVertex3f(ANCHO_MARCO+2*RADIO_MANIVELA,-1,ALTO_PUERTA/2);
      for(i=0;i<=CIRCLE_NUM_SECTORS;i++) {
          angle = i*2*PI/CIRCLE_NUM_SECTORS;
          glVertex3f(ANCHO_MARCO+2*RADIO_MANIVELA+RADIO_MANIVELA*cos(angle),
                     -1,
                     ALTO_PUERTA/2+RADIO_MANIVELA*sin(angle));
      }
    glEnd();       


    glBegin(GL_TRIANGLE_FAN);
      /* Manivela (otra cara): */
      glColor3ub(COLOR_MANIVELA);

      glVertex3f(ANCHO_MARCO+2*RADIO_MANIVELA,1,ALTO_PUERTA/2);
      for(i=0;i<=CIRCLE_NUM_SECTORS;i++) {
          angle = i*2*PI/CIRCLE_NUM_SECTORS;
          glVertex3f(ANCHO_MARCO+2*RADIO_MANIVELA+RADIO_MANIVELA*cos(angle),
                     1,
                     ALTO_PUERTA/2+RADIO_MANIVELA*sin(angle));
      }
    glEnd();       

}


void puerta3D(float x1,float y1,float x2,float y2)
{
    float angle,posx,posy,ancho;
    
    angle = atan2(y2-y1,x2-x1);
    posx = (x1+x2)/2;
    posy = (y1+y2)/2;
    ancho = sqrt((y2-y1)*(y2-y1)+(x2-x1)*(x2-x1));

    /*glPushMatrix();
    glRotatef(angle*180/PI,0,0,1);
    glTranslatef(posx*cos(angle)+posy*sin(angle),
                 -posx*sin(angle)+posy*cos(angle),0);
    puerta3D_internal(ancho); */

    glPushMatrix();
    //glTranslatef(-posx,0,0);
    glRotatef(angle*180/PI,0,0,1);
    glTranslatef(x1*cos(angle)+y1*sin(angle),
                 -x1*sin(angle)+y1*cos(angle),0);
    puerta3D_internal(ancho);


    glPopMatrix();
}


void pared3D(float x1,float y1,float x2,float y2)
{
    glBegin(GL_QUADS);       
      /* Cuadrado de pared encima del rodapie: */
      glColor3ub(COLOR_PARED);

      glVertex3f(x1,y1,ALTO_RODAPIE);
      glVertex3f(x1,y1,ALTO_PARED);
      glVertex3f(x2,y2,ALTO_PARED);
      glVertex3f(x2,y2,ALTO_RODAPIE);

      /* Rodapie: */
      glColor3ub(COLOR_RODAPIE);

      glVertex3f(x1,y1,0);
      glVertex3f(x1,y1,ALTO_RODAPIE);
      glVertex3f(x2,y2,ALTO_RODAPIE);
      glVertex3f(x2,y2,0);

    glEnd();
}


void suelo3D(float x1,float y1,float x2,float y2)
{
    glBegin(GL_QUADS);       
      glColor3ub(COLOR_SUELO);

      glVertex3f(x1,y1,-1);
      glVertex3f(x1,y2,-1);
      glVertex3f(x2,y2,-1);
      glVertex3f(x2,y1,-1);
    glEnd();
}


void geobot3D_internal(void)
{
    GLUquadricObj *obj;

    /* Creamos una cuadrica para el cilindro del cuerpo del robot, y para
     * las ruedas. */          
    obj = gluNewQuadric();
    gluQuadricDrawStyle(obj,GLU_FILL);
    gluQuadricNormals(obj,GLU_SMOOTH);
    gluQuadricOrientation(obj,GLU_OUTSIDE);    
    
    /* Cuerpo y tapa del robot: */
    glPushMatrix();
    glTranslatef(0,0,ELEVACION_ROBOT);
    glColor3ub(COLOR_CUERPO_ROBOT);
    gluCylinder(obj,RADIO_ROBOT,RADIO_ROBOT,ALTO_ROBOT,CIRCLE_NUM_SECTORS,1);
    glTranslatef(0,0,ALTO_ROBOT+1);
    glColor3ub(COLOR_TAPA_ROBOT);
    gluDisk(obj,0,RADIO_TAPA_ROBOT,CIRCLE_NUM_SECTORS,1);
    glPopMatrix();

    /* Ruedas: */
    glPushMatrix();
    glRotatef(90,0,1,0);
    glTranslatef(-RADIO_EXT_RUEDA_ROBOT,0,RADIO_ROBOT+1);
    glColor3ub(COLOR_RUEDAS_ROBOT);
    gluDisk(obj,RADIO_INT_RUEDA_ROBOT,RADIO_EXT_RUEDA_ROBOT,
                CIRCLE_NUM_SECTORS,1);
    glTranslatef(0,0,-2*(RADIO_ROBOT+1));
    gluDisk(obj,RADIO_INT_RUEDA_ROBOT,RADIO_EXT_RUEDA_ROBOT,
                CIRCLE_NUM_SECTORS,1);
    glPopMatrix();
    gluDeleteQuadric(obj);

    /* Tr�ode del robot: */
    glBegin(GL_LINES);
      glColor3ub(COLOR_TRIPODE_ROBOT);
      glVertex3f(0,-RADIO_ROBOT,ALTO_ROBOT+ELEVACION_ROBOT);
      glVertex3f(0,0,ALTO_ROBOT+ELEVACION_ROBOT+ALTO_TRIPODE);
     
      glVertex3f(SQR3DIV2*RADIO_ROBOT,RADIO_ROBOT/2,ALTO_ROBOT+ELEVACION_ROBOT);
      glVertex3f(0,0,ALTO_ROBOT+ELEVACION_ROBOT+ALTO_TRIPODE);

      glVertex3f(-SQR3DIV2*RADIO_ROBOT,RADIO_ROBOT/2,ALTO_ROBOT+ELEVACION_ROBOT);
      glVertex3f(0,0,ALTO_ROBOT+ELEVACION_ROBOT+ALTO_TRIPODE);    
    glEnd();    
    
    /* C�ara del robot: */
    /* Cuerpo de la c�ara: */
    glPushMatrix();
    glTranslatef(0,0,ALTO_ROBOT+ELEVACION_ROBOT+ALTO_TRIPODE);
    glRotatef(-90,1,0,0);
    glBegin(GL_QUAD_STRIP);
      glColor3ub(COLOR_CAMARA);
      glVertex3i(-CAM_WIDTH/2,-CAM_HEIGHT/2,-CAM_DEPTH/2);
      glVertex3i(-CAM_WIDTH/2,-CAM_HEIGHT/2,CAM_DEPTH/2);
      glVertex3i(CAM_WIDTH/2,-CAM_HEIGHT/2,-CAM_DEPTH/2);
      glVertex3i(CAM_WIDTH/2,-CAM_HEIGHT/2,CAM_DEPTH/2);
      glVertex3i(CAM_WIDTH/2,CAM_HEIGHT/2,-CAM_DEPTH/2);
      glVertex3i(CAM_WIDTH/2,CAM_HEIGHT/2,CAM_DEPTH/2);
      glVertex3i(-CAM_WIDTH/2,CAM_HEIGHT/2,-CAM_DEPTH/2);
      glVertex3i(-CAM_WIDTH/2,CAM_HEIGHT/2,CAM_DEPTH/2);
      glVertex3i(-CAM_WIDTH/2,-CAM_HEIGHT/2,-CAM_DEPTH/2);
      glVertex3i(-CAM_WIDTH/2,-CAM_HEIGHT/2,CAM_DEPTH/2);
    glEnd();     
    /* Ejes de la c�ara: */      
    glBegin(GL_LINES);
      glColor3ub(255,0,0);
      glVertex3i(0,0,0);
      glVertex3i((int)1.5*CAM_DEPTH,0,0);

      glColor3ub(0,255,0);
      glVertex3i(0,0,0);
      glVertex3i(0,(int)1.5*CAM_DEPTH,0); 

      glColor3ub(0,0,255);
      glVertex3i(0,0,0);
      glVertex3i(0,0,(int)1.5*CAM_DEPTH);                    
    glEnd(); 

    glPopMatrix();
}


void geobot3D(float posx, float posy, float angle)
{
    glPushMatrix();
    glRotatef(angle,0,0,1);
    glTranslatef(posx*cos(angle*PI/180)+posy*sin(angle*PI/180),
                 -posx*sin(angle*PI/180)+posy*cos(angle*PI/180),0);
    //glTranslatef(posx,posy,0);
    
    geobot3D_internal();
    glPopMatrix();
}



////////////////////////////////////////////////////////////////////////////


// devuelve un punto en el rayo que pasa por un pixel y a una distancia d del centro          
/*Matrix rayAt(VMatrix PIA, VMatrix c, VMatrix dir, double x, double y, double d) {
    Matrix r = PIA*CL(x,y,1.);
    r = CL(r(1)/r(4),r(2)/r(4),r(3)/r(4));
    r = r - c;
    if(DotProduct(r,dir)<0) {
        r = -1.*r;
    }
    r = r*(d/Norm(r));
    return c+r;
}*/


 
/*void parameterized_camera(LIST<double> p, float size, bool trip) {
    Requiere(p.size()==7,"problem in show_parameterized_camera");
    float x = p[4];
    float y = p[5];
    float z = p[6];
    Matrix A = SyntheticCamera(p);
    Matrix IA = PseudoInverse(A);
    if(z<0) {
        Debug("z negativo");
        return;
    }
    glPointSize(5.0);
    glBegin(GL_POINTS);
    glColor3ub(255,0,0);
    glVertex3f(x,y,z);                       
    glEnd();
    
    float d = Det3(ExtractArray(A,CL(1,2,3),CL(1,2,3))); // por si acaso
    float dx = d*A(3,1)/A(3,4);
    float dy = d*A(3,2)/A(3,4);                 
    float dz = d*A(3,3)/A(3,4);
    float n = sqrt(sqr(dx)+sqr(dy)+sqr(dz));
    Matrix dir = CL(double(dx/n),double(dy/n),double(dz/n));
    //glBegin(GL_LINE);
    //    glVertex3f(x,y,z);                 
    //    glVertex3f(x+size*dx/n,y+size*dy/n,z+size*dz/n);
    //glEnd();
    Matrix c = CL(double(x),double(y),double(z));
    
    LIST<Matrix> v;
    v.push_back(c);
    v.push_back(rayAt(IA,c,dir,-1.,-1.*3/4,size));
    v.push_back(rayAt(IA,c,dir,-1.,+1.*3/4,size));
    v.push_back(rayAt(IA,c,dir,+1.,+1.*3/4,size));
    v.push_back(rayAt(IA,c,dir,+1.,-1.*3/4,size));
    
#define VER(k) glVertex3f(v[k](1),v[k](2),v[k](3));
    
    //glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
    glColor3ub(255,255,255);
    glBegin(GL_POLYGON);    
    VER(1);
    VER(2);
    VER(3);
    VER(4);
    VER(1);
    glEnd();
    glColor3ub(255,0,0);
    glBegin(GL_LINE);
    VER(0); VER(1);
    VER(0); VER(2);
    VER(0); VER(3);
    VER(0); VER(4);
    VER(1); VER(2);
    VER(2); VER(3);
    VER(3); VER(4);
    VER(4); VER(1);
    glEnd();

    if(not trip) {
        return;
    }
    //dibujamos el trípode

    LIST<HPoint> T = Tripod(HomographyZ0(A));

    glColor3ub(40,40,0);
    glBegin(GL_LINE);
    VER(0); glVertex3f(T[0].x/T[0].w,T[0].y/T[0].w,0); 
    VER(0); glVertex3f(T[1].x/T[1].w,T[1].y/T[1].w,0); 
    VER(0); glVertex3f(T[2].x/T[2].w,T[2].y/T[2].w,0); 
    glEnd();

}

*/
