/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file tensor.cpp
@ingroup ExamplePrograms
@brief Tensor usage example program

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./tensor
\endcode

@author PARP Research Group. University of Murcia, Spain.
*/

#include <iostream>

/// \cond

#include <qtensor/qtensor.h>

///////////////////////////////////////////////////////////////////

int main ()
	{
	//for (int t = 0; t < 50000; t++)
		// Several operations
		{
		//QTensorIndex i = 5, j = 20, k = 800, l = 30, m = 60;
		QTensorIndex i = 3, j = 2, k = 8, l = 3, m = 6;
		QTensor	T( i * j.cov() * k );

		double *data = T.getWriteData();
		for (int n=0; n<T.getDataSize(); n++)
			data[n] = n;

		QTensor	A = T;
		QTensor	B = A.slice( i[1] & k.range(3,5) & k.range(0,2) & i[0] );
		QTensor	C = B.transpose(k * j.cov() * i);
		QTensorIndex i1 = 2, j1 = 3, k1 = 7;
		QTensor	D = C(i1 * j1.cov() * k1).slice(i1[1]);
 		QTensor E = D.transpose(k1, j1.cov());

		//std::cout << "B = A.slice( i[1] & k.range(3,5) & k.range(0,2) & i[0] ) = " << B << std::endl;
		//std::cout << "C = B.transpose(k * j.cov() * i) = " << C << std::endl;
		//std::cout << "D = C(i1 * j1.cov() * k1).slice(i1[1]) = " << D << std::endl;
		std::cout << "E = D.transpose(k1, j1.cov()) = " << E << std::endl;
		}

		// Contract	
		{
		QTensorIndex i1 = 2, j1 = 3, i2 = 2, j2 = 3;
		QTensor	T( i1 * j1.cov() * i2 * j2);
	
		double *data = T.getWriteData();
		for (int n=0; n<T.getDataSize(); n++)
			data[n] = n;
	
		QTensor	A = T(i1 * j1 * i1.cov() * j2);
		QTensor	B = T(i1 * j1 * i2 * j1.cov());
		QTensor	C = T(i1 * j1 * i1.cov() * j1.cov());

		std::cout << "T = " << T << std::endl;	
		std::cout << "A = T(i * j * i.cov() * l) = " << A << std::endl;
		std::cout << "B = T(i * j * k * j.cov()) = " << B << std::endl;
		std::cout << "C = T(i * j * i.cov() * j.cov()) = " << C << std::endl;
		}

		// Outer product
		{
		QTensorIndex i = 30, j = 16, k = 30;
		QTensor	T1(j*i);
		QTensor	T2(j.cov()*k);
	
		double *data1 = T1.getWriteData();
		for (int n=0; n<T1.getDataSize(); n++)
			data1[n] = n;

		double *data2 = T2.getWriteData();
		for (int n=0; n<T2.getDataSize(); n++)
			data2[n] = n;

		//std::cout << "T1 = " << T1 << std::endl;
		//std::cout << "T2 = " << T2 << std::endl;

		//for (int i=0; i < 1000; i++)
			{
			//QTensor Y = T1.outerProduct(T2).contract();
			QTensor Y2 = T1 * T2;
			//std::cout << "X =  = " << X << std::endl;
			//std::cout << "Y = T1.outerProduct(T2).X.contract() = " << Y << std::endl;
			//std::cout << "Y2 = T1(i * j) * T2(i.cov() * k) = " << Y2 << std::endl;
			}
		}

		}
/// \endcond
